@@ -9,6 +9,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+import typing as ty
+
 from openstack import exceptions
 from openstack import resource
 from openstack import utils
@@ -139,7 +142,7 @@ class Backup(resource.Resource):
         else:
             # Just for safety of the implementation (since PUT removed)
             raise exceptions.ResourceFailure(
-                "Invalid create method: %s" % self.create_method
+                f"Invalid create method: {self.create_method}"
             )
 
         has_body = (
@@ -173,14 +176,14 @@ class Backup(resource.Resource):
         :return: Updated backup instance
         """
         url = utils.urljoin(self.base_path, self.id, "restore")
-        body = {'restore': {}}
+        body: ty.Dict[str, ty.Dict] = {'restore': {}}
         if volume_id:
             body['restore']['volume_id'] = volume_id
         if name:
             body['restore']['name'] = name
         if not (volume_id or name):
             raise exceptions.SDKException(
-                'Either of `name` or `volume_id`' ' must be specified.'
+                'Either of `name` or `volume_id` must be specified.'
             )
         response = session.post(url, json=body)
         self._translate_response(response, has_body=False)
@@ -188,7 +191,7 @@ class Backup(resource.Resource):
 
     def force_delete(self, session):
         """Force backup deletion"""
-        body = {'os-force_delete': {}}
+        body = {'os-force_delete': None}
         self._action(session, body)
 
     def reset(self, session, status):
