@@ -18,6 +18,7 @@ Exception definitions.
 
 import json
 import re
+import typing as ty
 
 from requests import exceptions as _rex
 
@@ -28,24 +29,21 @@ class SDKException(Exception):
     def __init__(self, message=None, extra_data=None):
         self.message = self.__class__.__name__ if message is None else message
         self.extra_data = extra_data
-        super(SDKException, self).__init__(self.message)
-
-
-OpenStackCloudException = SDKException
+        super().__init__(self.message)
 
 
 class EndpointNotFound(SDKException):
     """A mismatch occurred between what the client and server expect."""
 
     def __init__(self, message=None):
-        super(EndpointNotFound, self).__init__(message)
+        super().__init__(message)
 
 
 class InvalidResponse(SDKException):
     """The response from the server is not valid for this request."""
 
     def __init__(self, response):
-        super(InvalidResponse, self).__init__()
+        super().__init__()
         self.response = response
 
 
@@ -53,10 +51,12 @@ class InvalidRequest(SDKException):
     """The request to the server is not valid."""
 
     def __init__(self, message=None):
-        super(InvalidRequest, self).__init__(message)
+        super().__init__(message)
 
 
 class HttpException(SDKException, _rex.HTTPError):
+    """The base exception for all HTTP error responses."""
+
     def __init__(
         self,
         message='Error',
@@ -69,13 +69,9 @@ class HttpException(SDKException, _rex.HTTPError):
         # to be None once we're not mocking Session everywhere.
         if not message:
             if response is not None:
-                message = "{name}: {code}".format(
-                    name=self.__class__.__name__, code=response.status_code
-                )
+                message = f"{self.__class__.__name__}: {response.status_code}"
             else:
-                message = "{name}: Unknown error".format(
-                    name=self.__class__.__name__
-                )
+                message = f"{self.__class__.__name__}: Unknown error"
 
         # Call directly rather than via super to control parameters
         SDKException.__init__(self, message=message)
@@ -102,43 +98,34 @@ class HttpException(SDKException, _rex.HTTPError):
         if not self.url or self.message == 'Error':
             return self.message
         if self.url:
-            remote_error = "{source} Error for url: {url}".format(
-                source=self.source, url=self.url
-            )
+            remote_error = f"{self.source} Error for url: {self.url}"
             if self.details:
                 remote_error += ', '
         if self.details:
             remote_error += str(self.details)
 
-        return "{message}: {remote_error}".format(
-            message=super(HttpException, self).__str__(),
-            remote_error=remote_error,
-        )
+        return f"{super().__str__()}: {remote_error}"
 
 
 class BadRequestException(HttpException):
     """HTTP 400 Bad Request."""
 
-    pass
+
+class NotFoundException(HttpException):
+    """HTTP 404 Not Found."""
 
 
 class ForbiddenException(HttpException):
     """HTTP 403 Forbidden Request."""
 
-    pass
-
 
 class ConflictException(HttpException):
     """HTTP 409 Conflict."""
 
-    pass
-
 
 class PreconditionFailedException(HttpException):
     """HTTP 412 Precondition Failed."""
 
-    pass
-
 
 class MethodNotSupported(SDKException):
     """The resource does not support this operation type."""
@@ -150,46 +137,25 @@ class MethodNotSupported(SDKException):
         except AttributeError:
             name = resource.__class__.__name__
 
-        message = 'The %s method is not supported for %s.%s' % (
-            method,
-            resource.__module__,
-            name,
-        )
-        super(MethodNotSupported, self).__init__(message=message)
+        message = f'The {method} method is not supported for {resource.__module__}.{name}'
+        super().__init__(message=message)
 
 
 class DuplicateResource(SDKException):
     """More than one resource exists with that name."""
 
-    pass
-
-
-class ResourceNotFound(HttpException):
-    """No resource exists with that name or id."""
-
-    pass
-
-
-NotFoundException = ResourceNotFound
-
 
 class ResourceTimeout(SDKException):
     """Timeout waiting for resource."""
 
-    pass
-
 
 class ResourceFailure(SDKException):
     """General resource failure."""
 
-    pass
-
 
 class InvalidResourceQuery(SDKException):
     """Invalid query params for resource."""
 
-    pass
-
 
 def _extract_message(obj):
     if isinstance(obj, dict):
@@ -214,6 +180,7 @@ def raise_from_response(response, error_message=None):
     if response.status_code < 400:
         return
 
+    cls: ty.Type[SDKException]
     if response.status_code == 400:
         cls = BadRequestException
     elif response.status_code == 403:
@@ -246,13 +213,14 @@ def raise_from_response(response, error_message=None):
         except Exception:
             details = response.text
     elif response.content and 'text/html' in content_type:
-        # Split the lines, strip whitespace and inline HTML from the response.
-        details = [
-            re.sub(r'<.+?>', '', i.strip()) for i in response.text.splitlines()
-        ]
-        details = list(set([msg for msg in details if msg]))
+        messages = []
+        for line in response.text.splitlines():
+            message = re.sub(r'<.+?>', '', line.strip())
+            if message not in messages:
+                messages.append(message)
+
         # Return joined string separated by colons.
-        details = ': '.join(details)
+        details = ': '.join(messages)
 
     if not details:
         details = response.reason if response.reason else response.text
@@ -287,3 +255,8 @@ class ServiceDisabledException(ConfigException):
 
 class ServiceDiscoveryException(SDKException):
     """The service cannot be discovered."""
+
+
+# Backwards compatibility
+OpenStackCloudException = SDKException
+ResourceNotFound = NotFoundException
