@@ -17,6 +17,7 @@ from openstack.orchestration.v1 import software_config as _sc
 from openstack.orchestration.v1 import software_deployment as _sd
 from openstack.orchestration.v1 import stack as _stack
 from openstack.orchestration.v1 import stack_environment as _stack_environment
+from openstack.orchestration.v1 import stack_event as _stack_event
 from openstack.orchestration.v1 import stack_files as _stack_files
 from openstack.orchestration.v1 import stack_template as _stack_template
 from openstack.orchestration.v1 import template as _template
@@ -24,6 +25,9 @@ from openstack import proxy
 from openstack import resource
 
 
+# TODO(rladntjr4): Some of these methods support lookup by ID, while
+#                  others support lookup by ID or name. We should choose one and use
+#                  it consistently.
 class Proxy(proxy.Proxy):
     _resource_registry = {
         "resource": _resource.Resource,
@@ -48,13 +52,13 @@ class Proxy(proxy.Proxy):
         if (
             url_parts[0] == 'stacks'
             and len(url_parts) > 2
-            and not url_parts[2] in ['preview', 'resources']
+            and url_parts[2] not in ['preview', 'resources']
         ):
             # orchestrate introduce having stack name and id part of the URL
             # (/stacks/name/id/everything_else), so if on third position we
             # have not a known part - discard it, not to brake further logic
             del url_parts[2]
-        return super(Proxy, self)._extract_name_consume_url_parts(url_parts)
+        return super()._extract_name_consume_url_parts(url_parts)
 
     def read_env_and_templates(
         self,
@@ -126,7 +130,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a stack.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -158,12 +162,12 @@ class Proxy(proxy.Proxy):
         :param resolve_outputs: Whether stack should contain outputs resolved.
 
         :returns: One :class:`~openstack.orchestration.v1.stack.Stack`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_stack.Stack, stack, resolve_outputs=resolve_outputs)
 
-    def update_stack(self, stack, preview=False, **attrs):
+    def update_stack(self, stack, *, preview=False, **attrs):
         """Update a stack
 
         :param stack: The value can be the ID of a stack or a
@@ -173,11 +177,11 @@ class Proxy(proxy.Proxy):
 
         :returns: The updated stack
         :rtype: :class:`~openstack.orchestration.v1.stack.Stack`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         res = self._get_resource(_stack.Stack, stack, **attrs)
-        return res.update(self, preview)
+        return res.commit(self, preview)
 
     def delete_stack(self, stack, ignore_missing=True):
         """Delete a stack
@@ -186,7 +190,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.orchestration.v1.stack.Stack`
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the stack does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent stack.
@@ -222,6 +226,41 @@ class Proxy(proxy.Proxy):
         res = self._get_resource(_stack.Stack, stack)
         return res.abandon(self)
 
+    def export_stack(self, stack):
+        """Get the stack data in JSON format
+
+        :param stack: The value can be the ID or a name or
+            an instance of :class:`~openstack.orchestration.v1.stack.Stack`
+        :returns: A dictionary containing the stack data.
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
+        """
+        if isinstance(stack, _stack.Stack):
+            obj = stack
+        else:
+            obj = self._find(_stack.Stack, stack, ignore_missing=False)
+        return obj.export(self)
+
+    def suspend_stack(self, stack):
+        """Suspend a stack status
+
+        :param stack: The value can be either the ID of a stack or an instance
+            of :class:`~openstack.orchestration.v1.stack.Stack`.
+        :returns: ``None``
+        """
+        res = self._get_resource(_stack.Stack, stack)
+        res.suspend(self)
+
+    def resume_stack(self, stack):
+        """Resume a stack status
+
+        :param stack: The value can be either the ID of a stack or an instance
+            of :class:`~openstack.orchestration.v1.stack.Stack`.
+        :returns: ``None``
+        """
+        res = self._get_resource(_stack.Stack, stack)
+        res.resume(self)
+
     def get_stack_template(self, stack):
         """Get template used by a stack
 
@@ -230,7 +269,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One object of
             :class:`~openstack.orchestration.v1.stack_template.StackTemplate`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         if isinstance(stack, _stack.Stack):
@@ -253,7 +292,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One object of
             :class:`~openstack.orchestration.v1.stack_environment.StackEnvironment`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         if isinstance(stack, _stack.Stack):
@@ -276,7 +315,7 @@ class Proxy(proxy.Proxy):
 
         :returns: A dictionary containing the names and contents of all files
             used by the stack.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when the stack cannot be found.
         """
         if isinstance(stack, _stack.Stack):
@@ -300,7 +339,7 @@ class Proxy(proxy.Proxy):
             an exception is thrown.
         :rtype: A generator of
             :class:`~openstack.orchestration.v1.resource.Resource`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when the stack cannot be found.
         """
         # first try treat the value as a stack object or an ID
@@ -356,7 +395,7 @@ class Proxy(proxy.Proxy):
             config or an instance of
             :class:`~openstack.orchestration.v1.software_config.SoftwareConfig`
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the software config does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent software config.
@@ -411,7 +450,7 @@ class Proxy(proxy.Proxy):
             software deployment or an instance of
             :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the software deployment does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent software deployment.
@@ -534,7 +573,7 @@ class Proxy(proxy.Proxy):
             )
         except Exception as e:
             raise exceptions.SDKException(
-                "Error in processing template files: %s" % str(e)
+                f"Error in processing template files: {str(e)}"
             )
 
     def _get_cleanup_dependencies(self):
@@ -549,7 +588,11 @@ class Proxy(proxy.Proxy):
         identified_resources=None,
         filters=None,
         resource_evaluation_fn=None,
+        skip_resources=None,
     ):
+        if self.should_skip_resource_cleanup("stack", skip_resources):
+            return
+
         stacks = []
         for obj in self.stacks():
             need_delete = self._service_cleanup_del_res(
@@ -566,3 +609,38 @@ class Proxy(proxy.Proxy):
 
         for stack in stacks:
             self.wait_for_delete(stack)
+
+    def stack_events(self, stack, resource_name=None, **attr):
+        """Get a stack events
+
+        :param stack: The value can be the ID of a stack or an instance of
+            :class:`~openstack.orchestration.v1.stack.Stack`
+        :param resource_name: The name of resource. If the resource_name is not None,
+            the base_path changes.
+
+        :returns: A generator of stack_events objects
+        :rtype: :class:`~openstack.orchestration.v1.stack_event.StackEvent`
+        """
+
+        if isinstance(stack, _stack.Stack):
+            obj = stack
+        else:
+            obj = self._get(_stack.Stack, stack)
+
+        if resource_name:
+            base_path = '/stacks/%(stack_name)s/%(stack_id)s/resources/%(resource_name)s/events'
+            return self._list(
+                _stack_event.StackEvent,
+                stack_name=obj.name,
+                stack_id=obj.id,
+                resource_name=resource_name,
+                base_path=base_path,
+                **attr,
+            )
+
+        return self._list(
+            _stack_event.StackEvent,
+            stack_name=obj.name,
+            stack_id=obj.id,
+            **attr,
+        )
