@@ -32,10 +32,12 @@ converted into this Resource class' appropriate components and types
 and then returned to the caller.
 """
 
+import abc
 import collections
 import inspect
 import itertools
 import operator
+import typing as ty
 import urllib.parse
 import warnings
 
@@ -93,11 +95,11 @@ def _convert_type(value, data_type, list_type=None):
             return data_type()
 
 
-class _BaseComponent:
+class _BaseComponent(abc.ABC):
     # The name this component is being tracked as in the Resource
-    key = None
+    key: str
     # The class to be used for mappings
-    _map_cls = dict
+    _map_cls: ty.Type[ty.Mapping] = dict
 
     #: Marks the property as deprecated.
     deprecated = False
@@ -219,8 +221,9 @@ class _BaseComponent:
 
         if value and deprecated:
             warnings.warn(
-                "The field %r has been deprecated. %s"
-                % (self.name, deprecation_reason or "Avoid usage."),
+                "The field {!r} has been deprecated. {}".format(
+                    self.name, deprecation_reason or "Avoid usage."
+                ),
                 os_warnings.RemovedFieldWarning,
             )
         return value
@@ -270,6 +273,8 @@ class Computed(_BaseComponent):
 class _ComponentManager(collections.abc.MutableMapping):
     """Storage of a component type"""
 
+    attributes: ty.Dict[str, ty.Any]
+
     def __init__(self, attributes=None, synchronized=False):
         self.attributes = dict() if attributes is None else attributes.copy()
         self._dirty = set() if synchronized else set(self.attributes.keys())
@@ -302,9 +307,7 @@ class _ComponentManager(collections.abc.MutableMapping):
     @property
     def dirty(self):
         """Return a dict of modified attributes"""
-        return dict(
-            (key, self.attributes.get(key, None)) for key in self._dirty
-        )
+        return {key: self.attributes.get(key, None) for key in self._dirty}
 
     def clean(self, only=None):
         """Signal that the resource no longer has modified attributes.
@@ -351,7 +354,7 @@ class QueryParameters:
             parameters, ``limit`` and ``marker``. These are the most common
             query parameters used for listing resources in OpenStack APIs.
         """
-        self._mapping = {}
+        self._mapping: ty.Dict[str, ty.Union[str, ty.Dict]] = {}
         if include_pagination_defaults:
             self._mapping.update({"limit": "limit", "marker": "marker"})
         self._mapping.update({name: name for name in names})
@@ -369,7 +372,7 @@ class QueryParameters:
 
         :returns: Filtered collection of the supported QueryParameters
         """
-        expected_params = list(self._mapping.keys())
+        expected_params = list(self._mapping)
         expected_params.extend(
             value.get('name', key) if isinstance(value, dict) else value
             for key, value in self._mapping.items()
@@ -378,20 +381,19 @@ class QueryParameters:
         if base_path:
             expected_params += utils.get_string_format_keys(base_path)
 
-        invalid_keys = set(query.keys()) - set(expected_params)
+        invalid_keys = set(query) - set(expected_params)
         if not invalid_keys:
             return query
         else:
             if not allow_unknown_params:
                 raise exceptions.InvalidResourceQuery(
-                    message="Invalid query params: %s"
-                    % ",".join(invalid_keys),
+                    message="Invalid query params: {}".format(
+                        ",".join(invalid_keys)
+                    ),
                     extra_data=invalid_keys,
                 )
             else:
-                known_keys = set(query.keys()).intersection(
-                    set(expected_params)
-                )
+                known_keys = set(query).intersection(set(expected_params))
                 return {k: query[k] for k in known_keys}
 
     def _transpose(self, query, resource_type):
@@ -452,27 +454,25 @@ class Resource(dict):
     # will work properly.
 
     #: Singular form of key for resource.
-    resource_key = None
+    resource_key: ty.Optional[str] = None
     #: Plural form of key for resource.
-    resources_key = None
+    resources_key: ty.Optional[str] = None
     #: Key used for pagination links
-    pagination_key = None
+    pagination_key: ty.Optional[str] = None
 
     #: The ID of this resource.
     id = Body("id")
+
     #: The name of this resource.
-    name = Body("name")
+    name: ty.Union[Body, URI] = Body("name")
     #: The OpenStack location of this resource.
-    location = Computed('location')
+    location: ty.Union[Computed, Body, Header] = Computed('location')
 
     #: Mapping of accepted query parameter names.
     _query_mapping = QueryParameters()
 
     #: The base part of the URI for this resource.
-    base_path = ""
-
-    #: The service associated with this resource to find the service URL.
-    service = None
+    base_path: str = ""
 
     #: Allow create operation for this resource.
     allow_create = False
@@ -502,31 +502,31 @@ class Resource(dict):
     #: Do calls for this resource require an id
     requires_id = True
     #: Whether create requires an ID (determined from method if None).
-    create_requires_id = None
+    create_requires_id: ty.Optional[bool] = None
     #: Whether create should exclude ID in the body of the request.
     create_exclude_id_from_body = False
     #: Do responses for this resource have bodies
     has_body = True
     #: Does create returns a body (if False requires ID), defaults to has_body
-    create_returns_body = None
+    create_returns_body: ty.Optional[bool] = None
 
     #: Maximum microversion to use for getting/creating/updating the Resource
-    _max_microversion = None
+    _max_microversion: ty.Optional[str] = None
     #: API microversion (string or None) this Resource was loaded with
     microversion = None
 
     _connection = None
-    _body = None
-    _header = None
-    _uri = None
-    _computed = None
-    _original_body = None
+    _body: _ComponentManager
+    _header: _ComponentManager
+    _uri: _ComponentManager
+    _computed: _ComponentManager
+    _original_body: ty.Dict[str, ty.Any] = {}
     _store_unknown_attrs_as_properties = False
     _allow_unknown_attrs_in_body = False
-    _unknown_attrs_in_body = None
+    _unknown_attrs_in_body: ty.Dict[str, ty.Any] = {}
 
     # Placeholder for aliases as dict of {__alias__:__original}
-    _attr_aliases = {}
+    _attr_aliases: ty.Dict[str, str] = {}
 
     def __init__(self, _synchronized=False, connection=None, **attrs):
         """The base resource
@@ -610,7 +610,7 @@ class Resource(dict):
 
     def __repr__(self):
         pairs = [
-            "%s=%s" % (k, v if v is not None else 'None')
+            "{}={}".format(k, v if v is not None else 'None')
             for k, v in dict(
                 itertools.chain(
                     self._body.attributes.items(),
@@ -622,7 +622,7 @@ class Resource(dict):
         ]
         args = ", ".join(pairs)
 
-        return "%s.%s(%s)" % (self.__module__, self.__class__.__name__, args)
+        return f"{self.__module__}.{self.__class__.__name__}({args})"
 
     def __eq__(self, comparand):
         """Return True if another resource has the same contents"""
@@ -687,9 +687,8 @@ class Resource(dict):
             for attr, component in self._attributes_iterator(tuple([Body])):
                 if component.name == name:
                     warnings.warn(
-                        "Access to '%s[%s]' is deprecated. "
-                        "Use '%s.%s' attribute instead"
-                        % (self.__class__, name, self.__class__, attr),
+                        f"Access to '{self.__class__}[{name}]' is deprecated. "
+                        f"Use '{self.__class__}.{attr}' attribute instead",
                         os_warnings.LegacyAPIWarning,
                     )
                     return getattr(self, attr)
@@ -710,13 +709,9 @@ class Resource(dict):
                 self._unknown_attrs_in_body[name] = value
                 return
             raise KeyError(
-                "{name} is not found. {module}.{cls} objects do not support"
-                " setting arbitrary keys through the"
-                " dict interface.".format(
-                    module=self.__module__,
-                    cls=self.__class__.__name__,
-                    name=name,
-                )
+                f"{name} is not found. "
+                f"{self.__module__}.{self.__class__.__name__} objects do not "
+                f"support setting arbitrary keys through the dict interface."
             )
 
     def _attributes(
@@ -1075,12 +1070,13 @@ class Resource(dict):
         :return: A dictionary of key/value pairs where keys are named
             as they exist as attributes of this class.
         """
+        mapping: ty.Union[utils.Munch, ty.Dict]
         if _to_munch:
             mapping = utils.Munch()
         else:
             mapping = {}
 
-        components = []
+        components: ty.List[ty.Type[_BaseComponent]] = []
         if body:
             components.append(Body)
         if headers:
@@ -1092,9 +1088,6 @@ class Resource(dict):
                 "At least one of `body`, `headers` or `computed` must be True"
             )
 
-        # isinstance stricly requires this to be a tuple
-        components = tuple(components)
-
         if body and self._allow_unknown_attrs_in_body:
             for key in self._unknown_attrs_in_body:
                 converted = self._attr_to_dict(
@@ -1108,7 +1101,8 @@ class Resource(dict):
         # but is slightly different in that we're looking at an instance
         # and we're mapping names on this class to their actual stored
         # values.
-        for attr, component in self._attributes_iterator(components):
+        # NOTE: isinstance stricly requires components to be a tuple
+        for attr, component in self._attributes_iterator(tuple(components)):
             if original_names:
                 key = component.name
             else:
@@ -1170,6 +1164,7 @@ class Resource(dict):
         *,
         resource_request_key=None,
     ):
+        body: ty.Union[ty.Dict[str, ty.Any], ty.List[ty.Any]]
         if patch:
             if not self._store_unknown_attrs_as_properties:
                 # Default case
@@ -1339,13 +1334,10 @@ class Resource(dict):
         # Resource class that is calling session.$something to be complete.
         if isinstance(session, adapter.Adapter):
             return session
-        if hasattr(session, '_sdk_connection'):
-            service_type = cls.service['service_type']
-            return getattr(session._sdk_connection, service_type)
         raise ValueError(
-            "The session argument to Resource methods requires either an"
-            " instance of an openstack.proxy.Proxy object or at the very least"
-            " a raw keystoneauth1.adapter.Adapter."
+            "The session argument to Resource methods requires either an "
+            "instance of an openstack.proxy.Proxy object or at the very least "
+            "a raw keystoneauth1.adapter.Adapter."
         )
 
     @classmethod
@@ -1376,7 +1368,7 @@ class Resource(dict):
             'delete',
             'patch',
         }:
-            raise ValueError('Invalid action: %s' % action)
+            raise ValueError(f'Invalid action: {action}')
 
         if session.default_microversion:
             return session.default_microversion
@@ -1409,7 +1401,7 @@ class Resource(dict):
         def _raise(message):
             if error_message:
                 error_message.rstrip('.')
-                message = '%s. %s' % (error_message, message)
+                message = f'{error_message}. {message}'
 
             raise exceptions.NotSupported(message)
 
@@ -1417,9 +1409,9 @@ class Resource(dict):
 
         if actual is None:
             message = (
-                "API version %s is required, but the default "
+                f"API version {expected} is required, but the default "
                 "version will be used."
-            ) % expected
+            )
             _raise(message)
         actual_n = discover.normalize_version_number(actual)
 
@@ -1427,9 +1419,9 @@ class Resource(dict):
             expected_n = discover.normalize_version_number(expected)
             if actual_n < expected_n:
                 message = (
-                    "API version %(expected)s is required, but %(actual)s "
+                    f"API version {expected} is required, but {actual} "
                     "will be used."
-                ) % {'expected': expected, 'actual': actual}
+                )
                 _raise(message)
         if maximum is not None:
             maximum_n = discover.normalize_version_number(maximum)
@@ -1517,7 +1509,7 @@ class Resource(dict):
             )
         else:
             raise exceptions.ResourceFailure(
-                "Invalid create method: %s" % self.create_method
+                f"Invalid create method: {self.create_method}"
             )
 
         has_body = (
@@ -1579,7 +1571,7 @@ class Resource(dict):
             and isinstance(data, list)
             and all([isinstance(x, dict) for x in data])
         ):
-            raise ValueError('Invalid data passed: %s' % data)
+            raise ValueError(f'Invalid data passed: {data}')
 
         session = cls._get_session(session)
         if microversion is None:
@@ -1595,10 +1587,10 @@ class Resource(dict):
             method = session.post
         else:
             raise exceptions.ResourceFailure(
-                "Invalid create method: %s" % cls.create_method
+                f"Invalid create method: {cls.create_method}"
             )
 
-        body = []
+        _body: ty.List[ty.Any] = []
         resources = []
         for attrs in data:
             # NOTE(gryf): we need to create resource objects, since
@@ -1611,9 +1603,12 @@ class Resource(dict):
             request = resource._prepare_request(
                 requires_id=requires_id, base_path=base_path
             )
-            body.append(request.body)
+            _body.append(request.body)
+
+        body: ty.Union[ty.Dict[str, ty.Any], ty.List[ty.Any]] = _body
 
         if prepend_key:
+            assert cls.resources_key
             body = {cls.resources_key: body}
 
         response = method(
@@ -1680,7 +1675,7 @@ class Resource(dict):
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
             :data:`Resource.allow_fetch` is not set to ``True``.
-        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
+        :raises: :exc:`~openstack.exceptions.NotFoundException` if
             the resource was not found.
         """
         if not self.allow_fetch:
@@ -1721,7 +1716,7 @@ class Resource(dict):
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
             :data:`Resource.allow_head` is not set to ``True``.
-        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if the resource
+        :raises: :exc:`~openstack.exceptions.NotFoundException` if the resource
             was not found.
         """
         if not self.allow_head:
@@ -1831,7 +1826,7 @@ class Resource(dict):
             call = getattr(session, method.lower())
         except AttributeError:
             raise exceptions.ResourceFailure(
-                "Invalid commit method: %s" % method
+                f"Invalid commit method: {method}"
             )
 
         response = call(
@@ -1858,7 +1853,7 @@ class Resource(dict):
                 parts = path.lstrip('/').split('/', 1)
                 field = parts[0]
             except (KeyError, IndexError):
-                raise ValueError("Malformed or missing path in %s" % item)
+                raise ValueError(f"Malformed or missing path in {item}")
 
             try:
                 component = getattr(self.__class__, field)
@@ -1868,9 +1863,9 @@ class Resource(dict):
                 server_field = component.name
 
             if len(parts) > 1:
-                new_path = '/%s/%s' % (server_field, parts[1])
+                new_path = f'/{server_field}/{parts[1]}'
             else:
-                new_path = '/%s' % server_field
+                new_path = f'/{server_field}'
             converted.append(dict(item, path=new_path))
 
         return converted
@@ -1952,7 +1947,7 @@ class Resource(dict):
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
             :data:`Resource.allow_commit` is not set to ``True``.
-        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
+        :raises: :exc:`~openstack.exceptions.NotFoundException` if
             the resource was not found.
         """
 
@@ -1990,6 +1985,7 @@ class Resource(dict):
         allow_unknown_params=False,
         *,
         microversion=None,
+        headers=None,
         **params,
     ):
         """This method is a generator which yields resource objects.
@@ -2010,6 +2006,8 @@ class Resource(dict):
             passing everything known to the server. ``False`` will result in
             validation exception when unknown query parameters are passed.
         :param str microversion: API version to override the negotiated one.
+        :param dict headers: Additional headers to inject into the HTTP
+            request.
         :param dict params: These keyword arguments are passed through the
             :meth:`~openstack.resource.QueryParamter._transpose` method
             to find if any of them match expected query parameters to be sent
@@ -2079,6 +2077,10 @@ class Resource(dict):
                     return False
             return True
 
+        headers_final = {"Accept": "application/json"}
+        if headers:
+            headers_final = {**headers_final, **headers}
+
         # Track the total number of resources yielded so we can paginate
         # swift objects
         total_yielded = 0
@@ -2086,7 +2088,7 @@ class Resource(dict):
             # Copy query_params due to weird mock unittest interactions
             response = session.get(
                 uri,
-                headers={"Accept": "application/json"},
+                headers=headers_final,
                 params=query_params.copy(),
                 microversion=microversion,
             )
@@ -2172,7 +2174,7 @@ class Resource(dict):
 
             if not pagination_key and cls.resources_key:
                 # Nova has a {key}_links dict in the main body
-                pagination_key = '{key}_links'.format(key=cls.resources_key)
+                pagination_key = f'{cls.resources_key}_links'
 
             if pagination_key:
                 links = data.get(pagination_key, {})
@@ -2246,16 +2248,60 @@ class Resource(dict):
 
         return the_result
 
+    @ty.overload
     @classmethod
     def find(
         cls,
         session,
-        name_or_id,
-        ignore_missing=True,
-        list_base_path=None,
+        name_or_id: str,
+        ignore_missing: ty.Literal[True] = True,
+        list_base_path: ty.Optional[str] = None,
         *,
-        microversion=None,
-        all_projects=None,
+        microversion: ty.Optional[str] = None,
+        all_projects: ty.Optional[bool] = None,
+        **params,
+    ) -> ty.Optional['Resource']: ...
+
+    @ty.overload
+    @classmethod
+    def find(
+        cls,
+        session,
+        name_or_id: str,
+        ignore_missing: ty.Literal[False],
+        list_base_path: ty.Optional[str] = None,
+        *,
+        microversion: ty.Optional[str] = None,
+        all_projects: ty.Optional[bool] = None,
+        **params,
+    ) -> 'Resource': ...
+
+    # excuse the duplication here: it's mypy's fault
+    # https://github.com/python/mypy/issues/14764
+    @ty.overload
+    @classmethod
+    def find(
+        cls,
+        session,
+        name_or_id: str,
+        ignore_missing: bool,
+        list_base_path: ty.Optional[str] = None,
+        *,
+        microversion: ty.Optional[str] = None,
+        all_projects: ty.Optional[bool] = None,
+        **params,
+    ): ...
+
+    @classmethod
+    def find(
+        cls,
+        session,
+        name_or_id: str,
+        ignore_missing: bool = True,
+        list_base_path: ty.Optional[str] = None,
+        *,
+        microversion: ty.Optional[str] = None,
+        all_projects: ty.Optional[bool] = None,
         **params,
     ):
         """Find a resource by its name or id.
@@ -2265,9 +2311,9 @@ class Resource(dict):
         :param name_or_id: This resource's identifier, if needed by
             the request. The default is ``None``.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
-            the resource does not exist.  When set to ``True``, None will be
-            returned when attempting to find a nonexistent resource.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the resource does not exist.  When set to ``True``, None will
+            be returned when attempting to find a nonexistent resource.
         :param str list_base_path: base_path to be used when need listing
             resources.
         :param str microversion: API version to override the negotiated one.
@@ -2280,8 +2326,8 @@ class Resource(dict):
             or None if nothing matches.
         :raises: :class:`openstack.exceptions.DuplicateResource` if more
             than one resource is found for this request.
-        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
-            is found and ignore_missing is ``False``.
+        :raises: :class:`openstack.exceptions.NotFoundException` if nothing is
+            found and ignore_missing is ``False``.
         """
         session = cls._get_session(session)
 
@@ -2326,8 +2372,8 @@ class Resource(dict):
         if ignore_missing:
             return None
 
-        raise exceptions.ResourceNotFound(
-            "No %s found for %s" % (cls.__name__, name_or_id)
+        raise exceptions.NotFoundException(
+            f"No {cls.__name__} found for {name_or_id}"
         )
 
 
@@ -2383,10 +2429,8 @@ def wait_for_status(
         failures = ['ERROR']
 
     failures = [f.lower() for f in failures]
-    name = "{res}:{id}".format(res=resource.__class__.__name__, id=resource.id)
-    msg = "Timeout waiting for {name} to transition to {status}".format(
-        name=name, status=status
-    )
+    name = f"{resource.__class__.__name__}:{resource.id}"
+    msg = f"Timeout waiting for {name} to transition to {status}"
 
     for count in utils.iterate_timeout(
         timeout=wait, message=msg, wait=interval
@@ -2394,9 +2438,7 @@ def wait_for_status(
         resource = resource.fetch(session, skip_cache=True)
         if not resource:
             raise exceptions.ResourceFailure(
-                "{name} went away while waiting for {status}".format(
-                    name=name, status=status
-                )
+                f"{name} went away while waiting for {status}"
             )
 
         new_status = getattr(resource, attribute)
@@ -2405,9 +2447,7 @@ def wait_for_status(
             return resource
         elif normalized_status in failures:
             raise exceptions.ResourceFailure(
-                "{name} transitioned to failure state {status}".format(
-                    name=name, status=new_status
-                )
+                f"{name} transitioned to failure state {new_status}"
             )
 
         LOG.debug(
@@ -2443,17 +2483,17 @@ def wait_for_delete(session, resource, interval, wait, callback=None):
     orig_resource = resource
     for count in utils.iterate_timeout(
         timeout=wait,
-        message="Timeout waiting for {res}:{id} to delete".format(
-            res=resource.__class__.__name__, id=resource.id
-        ),
+        message=f"Timeout waiting for {resource.__class__.__name__}:{resource.id} to delete",
         wait=interval,
     ):
         try:
             resource = resource.fetch(session, skip_cache=True)
             if not resource:
                 return orig_resource
-            if resource.status.lower() == 'deleted':
-                return resource
+            # Some resources like VolumeAttachment don't have status field.
+            if hasattr(resource, 'status'):
+                if resource.status.lower() == 'deleted':
+                    return resource
         except exceptions.NotFoundException:
             return orig_resource
 
