@@ -14,11 +14,13 @@
 import testtools
 from testtools import matchers
 
-from openstack.cloud import exc
+from openstack import exceptions
 from openstack.tests.unit import base
 
 
 class TestRoleAssignment(base.TestCase):
+    IS_INHERITED = False
+
     def _build_role_assignment_response(
         self, role_id, scope_type, scope_id, entity_type, entity_id
     ):
@@ -48,7 +50,7 @@ class TestRoleAssignment(base.TestCase):
         ]
 
     def setUp(self, cloud_config_fixture='clouds.yaml'):
-        super(TestRoleAssignment, self).setUp(cloud_config_fixture)
+        super().setUp(cloud_config_fixture)
         self.role_data = self._get_role_data()
         self.domain_data = self._get_domain_data()
         self.user_data = self._get_user_data(
@@ -119,8 +121,14 @@ class TestRoleAssignment(base.TestCase):
         append=None,
         base_url_append='v3',
         qs_elements=None,
+        inherited=False,
     ):
-        return super(TestRoleAssignment, self).get_mock_url(
+        if inherited:
+            base_url_append = base_url_append + '/OS-INHERIT'
+        if append and inherited:
+            append.append('inherited_to_projects')
+
+        return super().get_mock_url(
             service_type,
             interface,
             resource,
@@ -199,9 +207,11 @@ class TestRoleAssignment(base.TestCase):
                     uri=self.get_mock_url(resource='users'),
                     status_code=200,
                     json={
-                        'users': [user_data.json_response['user']]
-                        if is_found
-                        else []
+                        'users': (
+                            [user_data.json_response['user']]
+                            if is_found
+                            else []
+                        )
                     },
                 )
             )
@@ -215,9 +225,11 @@ class TestRoleAssignment(base.TestCase):
                     ),
                     status_code=200,
                     json={
-                        'users': [user_data.json_response['user']]
-                        if is_found
-                        else []
+                        'users': (
+                            [user_data.json_response['user']]
+                            if is_found
+                            else []
+                        )
                     },
                 )
             )
@@ -314,6 +326,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -329,6 +342,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -341,6 +355,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -366,6 +381,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -381,6 +397,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -393,6 +410,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -415,6 +433,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -428,6 +447,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -453,6 +473,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -466,6 +487,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -490,6 +512,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -505,6 +528,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -517,6 +541,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -542,6 +567,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -557,6 +583,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -569,6 +596,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -592,6 +620,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -605,6 +634,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -630,6 +660,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -643,6 +674,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -668,6 +700,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -683,6 +716,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -695,6 +729,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -720,6 +755,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -735,6 +771,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -747,6 +784,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -769,6 +807,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -782,6 +821,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -807,6 +847,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -820,6 +861,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -844,6 +886,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -859,6 +902,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -871,6 +915,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -896,6 +941,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -911,6 +957,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -923,6 +970,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -946,6 +994,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -959,6 +1008,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -984,6 +1034,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -997,6 +1048,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1022,6 +1074,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1037,6 +1090,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1049,6 +1103,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1074,6 +1129,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1089,6 +1145,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1101,6 +1158,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -1123,6 +1181,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1136,6 +1195,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1161,6 +1221,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1174,6 +1235,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1198,6 +1260,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1213,6 +1276,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1225,6 +1289,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1250,6 +1315,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1265,6 +1331,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1277,6 +1344,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1300,6 +1368,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1313,6 +1382,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1338,6 +1408,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1351,6 +1422,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1376,6 +1448,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1391,6 +1464,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1403,6 +1477,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1428,6 +1503,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1443,6 +1519,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1455,6 +1532,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -1477,6 +1555,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1490,6 +1569,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1515,6 +1595,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1528,6 +1609,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1552,6 +1634,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1567,6 +1650,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1579,6 +1663,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1604,6 +1689,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1619,6 +1705,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1631,6 +1718,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1654,6 +1742,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1667,6 +1756,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1692,6 +1782,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1705,6 +1796,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1738,14 +1830,12 @@ class TestRoleAssignment(base.TestCase):
         )
         self.register_uris(uris)
 
-        with testtools.ExpectedException(
-            exc.OpenStackCloudException,
-            'Role {0} not found'.format(self.role_data.role_name),
-        ):
+        with testtools.ExpectedException(exceptions.NotFoundException):
             self.cloud.grant_role(
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1778,14 +1868,12 @@ class TestRoleAssignment(base.TestCase):
         )
         self.register_uris(uris)
 
-        with testtools.ExpectedException(
-            exc.OpenStackCloudException,
-            'Role {0} not found'.format(self.role_data.role_name),
-        ):
+        with testtools.ExpectedException(exceptions.NotFoundException):
             self.cloud.revoke_role(
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1795,10 +1883,13 @@ class TestRoleAssignment(base.TestCase):
         )
         self.register_uris(uris)
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Must specify either a user or a group',
         ):
-            self.cloud.grant_role(self.role_data.role_name)
+            self.cloud.grant_role(
+                self.role_data.role_name,
+                inherited=self.IS_INHERITED,
+            )
         self.assert_calls()
 
     def test_revoke_no_user_or_group_specified(self):
@@ -1807,10 +1898,13 @@ class TestRoleAssignment(base.TestCase):
         )
         self.register_uris(uris)
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Must specify either a user or a group',
         ):
-            self.cloud.revoke_role(self.role_data.role_name)
+            self.cloud.revoke_role(
+                self.role_data.role_name,
+                inherited=self.IS_INHERITED,
+            )
         self.assert_calls()
 
     def test_grant_no_user_or_group(self):
@@ -1823,11 +1917,13 @@ class TestRoleAssignment(base.TestCase):
         self.register_uris(uris)
 
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Must specify either a user or a group',
         ):
             self.cloud.grant_role(
-                self.role_data.role_name, user=self.user_data.name
+                self.role_data.role_name,
+                user=self.user_data.name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1841,11 +1937,13 @@ class TestRoleAssignment(base.TestCase):
         self.register_uris(uris)
 
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Must specify either a user or a group',
         ):
             self.cloud.revoke_role(
-                self.role_data.role_name, user=self.user_data.name
+                self.role_data.role_name,
+                user=self.user_data.name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1862,13 +1960,14 @@ class TestRoleAssignment(base.TestCase):
         self.register_uris(uris)
 
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Specify either a group or a user, not both',
         ):
             self.cloud.grant_role(
                 self.role_data.role_name,
                 user=self.user_data.name,
                 group=self.group_data.group_name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1885,13 +1984,14 @@ class TestRoleAssignment(base.TestCase):
         self.register_uris(uris)
 
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Specify either a group or a user, not both',
         ):
             self.cloud.revoke_role(
                 self.role_data.role_name,
                 user=self.user_data.name,
                 group=self.group_data.group_name,
+                inherited=self.IS_INHERITED,
             )
 
     def test_grant_both_project_and_domain(self):
@@ -1919,6 +2019,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1934,6 +2035,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1947,6 +2049,7 @@ class TestRoleAssignment(base.TestCase):
                 user=self.user_data.name,
                 project=self.project_data.project_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -1975,6 +2078,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1990,6 +2094,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -2003,6 +2108,7 @@ class TestRoleAssignment(base.TestCase):
                 user=self.user_data.name,
                 project=self.project_data.project_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -2017,11 +2123,13 @@ class TestRoleAssignment(base.TestCase):
         self.register_uris(uris)
 
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Must specify either a domain, project or system',
         ):
             self.cloud.grant_role(
-                self.role_data.role_name, user=self.user_data.name
+                self.role_data.role_name,
+                user=self.user_data.name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -2036,11 +2144,13 @@ class TestRoleAssignment(base.TestCase):
         self.register_uris(uris)
 
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             'Must specify either a domain, project or system',
         ):
             self.cloud.revoke_role(
-                self.role_data.role_name, user=self.user_data.name
+                self.role_data.role_name,
+                user=self.user_data.name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -2063,11 +2173,12 @@ class TestRoleAssignment(base.TestCase):
                 ),
             ]
         )
-        with testtools.ExpectedException(exc.OpenStackCloudURINotFound):
+        with testtools.ExpectedException(exceptions.NotFoundException):
             self.cloud.grant_role(
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain='baddomain',
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -2090,10 +2201,15 @@ class TestRoleAssignment(base.TestCase):
                 ),
             ]
         )
-        with testtools.ExpectedException(exc.OpenStackCloudURINotFound):
+        with testtools.ExpectedException(exceptions.NotFoundException):
             self.cloud.revoke_role(
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain='baddomain',
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
+
+
+class TestInheritedRoleAssignment(TestRoleAssignment):
+    IS_INHERITED = True
