@@ -26,6 +26,7 @@ from openstack.compute.v2 import hypervisor
 from openstack.compute.v2 import image
 from openstack.compute.v2 import keypair
 from openstack.compute.v2 import migration
+from openstack.compute.v2 import quota_class_set
 from openstack.compute.v2 import quota_set
 from openstack.compute.v2 import server
 from openstack.compute.v2 import server_action
@@ -37,13 +38,15 @@ from openstack.compute.v2 import server_remote_console
 from openstack.compute.v2 import service
 from openstack.compute.v2 import usage
 from openstack.compute.v2 import volume_attachment
-from openstack import resource
+from openstack.identity.v3 import project
+from openstack import proxy as proxy_base
 from openstack.tests.unit import test_proxy_base
+from openstack import warnings as os_warnings
 
 
 class TestComputeProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestComputeProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -523,7 +526,10 @@ class TestVolumeAttachment(TestComputeProxy):
             )
 
             self.assertEqual(1, len(w))
-            self.assertEqual(w[-1].category, DeprecationWarning)
+            self.assertEqual(
+                os_warnings.RemovedInSDK50Warning,
+                w[-1].category,
+            )
             self.assertIn(
                 'This method was called with a volume_id or volumeId argument',
                 str(w[-1]),
@@ -957,6 +963,30 @@ class TestCompute(TestComputeProxy):
     def test_server_update(self):
         self.verify_update(self.proxy.update_server, server.Server)
 
+    def test_server_change_password(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.change_password",
+            self.proxy.change_server_password,
+            method_args=["value", "password"],
+            expected_args=[self.proxy, "password"],
+        )
+
+    def test_server_get_password(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.get_password",
+            self.proxy.get_server_password,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_server_clear_password(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.clear_password",
+            self.proxy.clear_server_password,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
     def test_server_wait_for(self):
         value = server.Server(id='1234')
         self.verify_wait_for_status(
@@ -1177,7 +1207,12 @@ class TestCompute(TestComputeProxy):
             self.proxy.evacuate_server,
             method_args=["value"],
             expected_args=[self.proxy],
-            expected_kwargs={"host": None, "admin_pass": None, "force": None},
+            expected_kwargs={
+                "host": None,
+                "admin_pass": None,
+                "force": None,
+                "on_shared_storage": None,
+            },
         )
 
     def test_server_evacuate_with_options(self):
@@ -1185,11 +1220,13 @@ class TestCompute(TestComputeProxy):
             "openstack.compute.v2.server.Server.evacuate",
             self.proxy.evacuate_server,
             method_args=["value", 'HOST2', 'NEW_PASS', True],
+            method_kwargs={'on_shared_storage': False},
             expected_args=[self.proxy],
             expected_kwargs={
                 "host": "HOST2",
                 "admin_pass": 'NEW_PASS',
                 "force": True,
+                "on_shared_storage": False,
             },
         )
 
@@ -1239,6 +1276,21 @@ class TestCompute(TestComputeProxy):
             self.proxy.unshelve_server,
             method_args=["value"],
             expected_args=[self.proxy],
+            expected_kwargs={
+                "host": None,
+            },
+        )
+
+    def test_server_unshelve_with_options(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.unshelve",
+            self.proxy.unshelve_server,
+            method_args=["value"],
+            method_kwargs={"host": "HOST2"},
+            expected_args=[self.proxy],
+            expected_kwargs={
+                "host": "HOST2",
+            },
         )
 
     def test_server_trigger_dump(self):
@@ -1249,6 +1301,30 @@ class TestCompute(TestComputeProxy):
             expected_args=[self.proxy],
         )
 
+    def test_server_add_tag(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.add_tag",
+            self.proxy.add_tag_to_server,
+            method_args=["value", "tag"],
+            expected_args=[self.proxy, "tag"],
+        )
+
+    def test_server_remove_tag(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.remove_tag",
+            self.proxy.remove_tag_from_server,
+            method_args=["value", "tag"],
+            expected_args=[self.proxy, "tag"],
+        )
+
+    def test_server_remove_tags(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.remove_all_tags",
+            self.proxy.remove_tags_from_server,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
     def test_get_server_output(self):
         self._verify(
             "openstack.compute.v2.server.Server.get_console_output",
@@ -1371,9 +1447,15 @@ class TestCompute(TestComputeProxy):
         self._verify(
             'openstack.compute.v2.server.Server.live_migrate',
             self.proxy.live_migrate_server,
-            method_args=["value", "host1", False],
-            expected_args=[self.proxy, "host1"],
-            expected_kwargs={'force': False, 'block_migration': None},
+            method_args=["value"],
+            method_kwargs={'host': 'host1', 'force': False},
+            expected_args=[self.proxy],
+            expected_kwargs={
+                'host': 'host1',
+                'force': False,
+                'block_migration': None,
+                'disk_over_commit': None,
+            },
         )
 
     def test_abort_server_migration(self):
@@ -1446,7 +1528,7 @@ class TestCompute(TestComputeProxy):
         self._verify(
             'openstack.compute.v2.server.Server.add_security_group',
             self.proxy.add_security_group_to_server,
-            method_args=["value", {'id': 'id', 'name': 'sg'}],
+            method_args=["value", 'sg'],
             expected_args=[self.proxy, 'sg'],
         )
 
@@ -1454,7 +1536,7 @@ class TestCompute(TestComputeProxy):
         self._verify(
             'openstack.compute.v2.server.Server.remove_security_group',
             self.proxy.remove_security_group_from_server,
-            method_args=["value", {'id': 'id', 'name': 'sg'}],
+            method_args=["value", 'sg'],
             expected_args=[self.proxy, 'sg'],
         )
 
@@ -1558,51 +1640,80 @@ class TestCompute(TestComputeProxy):
         self.assertEqual(console_fake['url'], ret['url'])
 
 
-class TestQuota(TestComputeProxy):
-    def test_get(self):
+class TestQuotaClassSet(TestComputeProxy):
+    def test_quota_class_set_get(self):
+        self.verify_get(
+            self.proxy.get_quota_class_set, quota_class_set.QuotaClassSet
+        )
+
+    def test_quota_class_set_update(self):
+        self.verify_update(
+            self.proxy.update_quota_class_set,
+            quota_class_set.QuotaClassSet,
+            False,
+        )
+
+
+class TestQuotaSet(TestComputeProxy):
+    def test_quota_set_get(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
             method_args=['prj'],
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                None,
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
-                'usage': False,
+                'microversion': None,
+                'resource_response_key': None,
             },
             method_result=quota_set.QuotaSet(),
             expected_result=quota_set.QuotaSet(),
         )
 
-    def test_get_query(self):
+    def test_quota_set_get_query(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
             method_args=['prj'],
             method_kwargs={'usage': True, 'user_id': 'uid'},
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                '/os-quota-sets/%(project_id)s/detail',
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
-                'usage': True,
+                'microversion': None,
+                'resource_response_key': None,
                 'user_id': 'uid',
             },
         )
 
-    def test_get_defaults(self):
+    def test_quota_set_get_defaults(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set_defaults,
             method_args=['prj'],
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                '/os-quota-sets/%(project_id)s/defaults',
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
-                'base_path': '/os-quota-sets/defaults',
+                'microversion': None,
+                'resource_response_key': None,
             },
         )
 
-    def test_reset(self):
+    def test_quota_set_reset(self):
         self._verify(
             'openstack.resource.Resource.delete',
             self.proxy.revert_quota_set,
@@ -1612,22 +1723,26 @@ class TestQuota(TestComputeProxy):
             expected_kwargs={'user_id': 'uid'},
         )
 
-    @mock.patch('openstack.proxy.Proxy._get_resource', autospec=True)
-    def test_update(self, gr_mock):
-        gr_mock.return_value = resource.Resource()
-        gr_mock.commit = mock.Mock()
+    @mock.patch.object(proxy_base.Proxy, "_get_resource")
+    def test_quota_set_update(self, mock_get):
+        fake_project = project.Project(id='prj')
+        fake_quota_set = quota_set.QuotaSet(project_id='prj')
+        mock_get.side_effect = [fake_project, fake_quota_set]
+
         self._verify(
             'openstack.resource.Resource.commit',
             self.proxy.update_quota_set,
-            method_args=['qs'],
-            method_kwargs={
-                'query': {'user_id': 'uid'},
-                'a': 'b',
-            },
+            method_args=['prj'],
+            method_kwargs={'ram': 123},
             expected_args=[self.proxy],
-            expected_kwargs={'user_id': 'uid'},
+            expected_kwargs={},
+        )
+        mock_get.assert_has_calls(
+            [
+                mock.call(project.Project, 'prj'),
+                mock.call(quota_set.QuotaSet, None, project_id='prj', ram=123),
+            ]
         )
-        gr_mock.assert_called_with(self.proxy, quota_set.QuotaSet, 'qs', a='b')
 
 
 class TestServerAction(TestComputeProxy):
