@@ -14,8 +14,11 @@ from unittest import mock
 
 from openstack.shared_file_system.v2 import _proxy
 from openstack.shared_file_system.v2 import limit
+from openstack.shared_file_system.v2 import resource_locks
 from openstack.shared_file_system.v2 import share
 from openstack.shared_file_system.v2 import share_access_rule
+from openstack.shared_file_system.v2 import share_group
+from openstack.shared_file_system.v2 import share_group_snapshot
 from openstack.shared_file_system.v2 import share_instance
 from openstack.shared_file_system.v2 import share_network
 from openstack.shared_file_system.v2 import share_network_subnet
@@ -28,7 +31,7 @@ from openstack.tests.unit import test_proxy_base
 
 class TestSharedFileSystemProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestSharedFileSystemProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -128,7 +131,7 @@ class TestSharedFileSystemShare(TestSharedFileSystemProxy):
         self.proxy.wait_for_status(mock_resource, 'ACTIVE')
 
         mock_wait.assert_called_once_with(
-            self.proxy, mock_resource, 'ACTIVE', [], 2, 120
+            self.proxy, mock_resource, 'ACTIVE', [], 2, 120, attribute='status'
         )
 
 
@@ -153,9 +156,66 @@ class TestSharedFileSystemStoragePool(TestSharedFileSystemProxy):
         )
 
 
+class TestSharedFileSystemShareMetadata(TestSharedFileSystemProxy):
+    def test_get_share_metadata(self):
+        self._verify(
+            "openstack.shared_file_system.v2.share.Share.fetch_metadata",
+            self.proxy.get_share_metadata,
+            method_args=["share_id"],
+            expected_args=[self.proxy],
+            expected_result=share.Share(
+                id="share_id", metadata={"key": "value"}
+            ),
+        )
+
+    def test_get_share_metadata_item(self):
+        self._verify(
+            "openstack.shared_file_system.v2.share.Share.get_metadata_item",
+            self.proxy.get_share_metadata_item,
+            method_args=["share_id", "key"],
+            expected_args=[self.proxy, "key"],
+            expected_result=share.Share(
+                id="share_id", metadata={"key": "value"}
+            ),
+        )
+
+    def test_create_share_metadata(self):
+        metadata = {"foo": "bar", "newFoo": "newBar"}
+        self._verify(
+            "openstack.shared_file_system.v2.share.Share.set_metadata",
+            self.proxy.create_share_metadata,
+            method_args=["share_id"],
+            method_kwargs=metadata,
+            expected_args=[self.proxy],
+            expected_kwargs={"metadata": metadata},
+            expected_result=share.Share(id="share_id", metadata=metadata),
+        )
+
+    def test_update_share_metadata(self):
+        metadata = {"foo": "bar", "newFoo": "newBar"}
+        replace = True
+        self._verify(
+            "openstack.shared_file_system.v2.share.Share.set_metadata",
+            self.proxy.update_share_metadata,
+            method_args=["share_id", metadata, replace],
+            expected_args=[self.proxy],
+            expected_kwargs={"metadata": metadata, "replace": replace},
+            expected_result=share.Share(id="share_id", metadata=metadata),
+        )
+
+    def test_delete_share_metadata(self):
+        self._verify(
+            "openstack.shared_file_system.v2.share.Share.delete_metadata_item",
+            self.proxy.delete_share_metadata,
+            expected_result=None,
+            method_args=["share_id", ["key"]],
+            expected_args=[self.proxy, "key"],
+        )
+
+
 class TestUserMessageProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestUserMessageProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
     def test_user_messages(self):
@@ -188,7 +248,7 @@ class TestUserMessageProxy(test_proxy_base.TestProxyBase):
 
 class TestShareSnapshotResource(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestShareSnapshotResource, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
     def test_share_snapshots(self):
@@ -253,7 +313,7 @@ class TestShareSnapshotResource(test_proxy_base.TestProxyBase):
 
 class TestShareSnapshotInstanceResource(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestShareSnapshotInstanceResource, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
     def test_share_snapshot_instances(self):
@@ -287,7 +347,7 @@ class TestShareSnapshotInstanceResource(test_proxy_base.TestProxyBase):
 
 class TestShareNetworkResource(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestShareNetworkResource, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
     def test_share_networks(self):
@@ -337,7 +397,7 @@ class TestShareNetworkResource(test_proxy_base.TestProxyBase):
 
 class TestShareNetworkSubnetResource(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestShareNetworkSubnetResource, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
     def test_share_network_subnets(self):
@@ -384,7 +444,7 @@ class TestShareNetworkSubnetResource(test_proxy_base.TestProxyBase):
 
 class TestAccessRuleProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestAccessRuleProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
     def test_access_ruless(self):
@@ -411,9 +471,123 @@ class TestAccessRuleProxy(test_proxy_base.TestProxyBase):
 
     def test_access_rules_delete(self):
         self._verify(
-            "openstack.shared_file_system.v2.share_access_rule."
-            + "ShareAccessRule.delete",
+            "openstack.shared_file_system.v2.share_access_rule.ShareAccessRule.delete",
             self.proxy.delete_access_rule,
-            method_args=['access_id', 'share_id', 'ignore_missing'],
-            expected_args=[self.proxy, 'share_id'],
+            method_args=[
+                'access_id',
+                'share_id',
+            ],
+            expected_args=[self.proxy],
+            expected_kwargs={'unrestrict': False},
+        )
+
+
+class TestResourceLocksProxy(test_proxy_base.TestProxyBase):
+    def setUp(self):
+        super().setUp()
+        self.proxy = _proxy.Proxy(self.session)
+
+    def test_list_resource_locks(self):
+        self.verify_list(
+            self.proxy.resource_locks, resource_locks.ResourceLock
+        )
+
+    def test_resource_lock_get(self):
+        self.verify_get(
+            self.proxy.get_resource_lock, resource_locks.ResourceLock
+        )
+
+    def test_resource_lock_delete(self):
+        self.verify_delete(
+            self.proxy.delete_resource_lock, resource_locks.ResourceLock, False
+        )
+
+    def test_resource_lock_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_resource_lock, resource_locks.ResourceLock, True
+        )
+
+    def test_resource_lock_create(self):
+        self.verify_create(
+            self.proxy.create_resource_lock, resource_locks.ResourceLock
+        )
+
+    def test_resource_lock_update(self):
+        self.verify_update(
+            self.proxy.update_resource_lock, resource_locks.ResourceLock
+        )
+
+
+class TestShareGroupResource(test_proxy_base.TestProxyBase):
+    def setUp(self):
+        super().setUp()
+        self.proxy = _proxy.Proxy(self.session)
+
+    def test_share_groups(self):
+        self.verify_list(self.proxy.share_groups, share_group.ShareGroup)
+
+    def test_share_groups_query(self):
+        self.verify_list(
+            self.proxy.share_groups,
+            share_group.ShareGroup,
+            method_kwargs={"query": 1},
+            expected_kwargs={"query": 1},
+        )
+
+    def test_share_group_get(self):
+        self.verify_get(self.proxy.get_share_group, share_group.ShareGroup)
+
+    def test_share_group_find(self):
+        self.verify_find(self.proxy.find_share_group, share_group.ShareGroup)
+
+    def test_share_group_delete(self):
+        self.verify_delete(
+            self.proxy.delete_share_group, share_group.ShareGroup, False
+        )
+
+    def test_share_group_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_share_group, share_group.ShareGroup, True
+        )
+
+    def test_share_group_create(self):
+        self.verify_create(
+            self.proxy.create_share_group, share_group.ShareGroup
+        )
+
+    def test_share_group_update(self):
+        self.verify_update(
+            self.proxy.update_share_group, share_group.ShareGroup
+        )
+
+    def test_share_group_snapshots(self):
+        self.verify_list(
+            self.proxy.share_group_snapshots,
+            share_group_snapshot.ShareGroupSnapshot,
+        )
+
+    def test_share_group_snapshot_get(self):
+        self.verify_get(
+            self.proxy.get_share_group_snapshot,
+            share_group_snapshot.ShareGroupSnapshot,
+        )
+
+    def test_share_group_snapshot_update(self):
+        self.verify_update(
+            self.proxy.update_share_group_snapshot,
+            share_group_snapshot.ShareGroupSnapshot,
+        )
+
+    def test_share_group_snapshot_delete(self):
+        self.verify_delete(
+            self.proxy.delete_share_group_snapshot,
+            share_group_snapshot.ShareGroupSnapshot,
+            False,
+        )
+
+    def test_share_group_snapshot_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_share_group_snapshot,
+            share_group_snapshot.ShareGroupSnapshot,
+            True,
         )
