@@ -16,6 +16,7 @@ import queue
 import string
 import threading
 import time
+import typing as ty
 
 import keystoneauth1
 from keystoneauth1 import adapter as ks_adapter
@@ -58,8 +59,8 @@ def iterate_timeout(timeout, message, wait=2):
         wait = float(wait)
     except ValueError:
         raise exceptions.SDKException(
-            "Wait value must be an int or float value. {wait} given"
-            " instead".format(wait=wait)
+            f"Wait value must be an int or float value. "
+            f"{wait} given instead"
         )
 
     start = time.time()
@@ -72,6 +73,16 @@ def iterate_timeout(timeout, message, wait=2):
     raise exceptions.ResourceTimeout(message)
 
 
+class _AccessSaver:
+    __slots__ = ('keys',)
+
+    def __init__(self):
+        self.keys = []
+
+    def __getitem__(self, key):
+        self.keys.append(key)
+
+
 def get_string_format_keys(fmt_string, old_style=True):
     """Gets a list of required keys from a format string
 
@@ -79,15 +90,7 @@ def get_string_format_keys(fmt_string, old_style=True):
     use the old style string formatting.
     """
     if old_style:
-
-        class AccessSaver:
-            def __init__(self):
-                self.keys = []
-
-            def __getitem__(self, key):
-                self.keys.append(key)
-
-        a = AccessSaver()
+        a = _AccessSaver()
         fmt_string % a
 
         return a.keys
@@ -169,17 +172,15 @@ def supports_microversion(adapter, microversion, raise_exception=False):
             supports = discover.version_match(required, candidate)
             if raise_exception and not supports:
                 raise exceptions.SDKException(
-                    'Required microversion {ver} is higher than currently '
-                    'selected {curr}'.format(
-                        ver=microversion, curr=adapter.default_microversion
-                    )
+                    f'Required microversion {microversion} is higher than '
+                    f'currently selected {adapter.default_microversion}'
                 )
             return supports
         return True
     if raise_exception:
         raise exceptions.SDKException(
-            'Required microversion {ver} is not supported '
-            'by the server side'.format(ver=microversion)
+            f'Required microversion {microversion} is not supported '
+            f'by the server side'
         )
     return False
 
@@ -284,11 +285,11 @@ def maximum_supported_microversion(adapter, client_maximum):
 
 
 def _hashes_up_to_date(md5, sha256, md5_key, sha256_key):
-    '''Compare md5 and sha256 hashes for being up to date
+    """Compare md5 and sha256 hashes for being up to date
 
     md5 and sha256 are the current values.
     md5_key and sha256_key are the previous values.
-    '''
+    """
     up_to_date = False
     if md5 and md5_key == md5:
         up_to_date = True
@@ -301,29 +302,8 @@ def _hashes_up_to_date(md5, sha256, md5_key, sha256_key):
     return up_to_date
 
 
-try:
-    _test_md5 = hashlib.md5(usedforsecurity=False)  # nosec
-
-    # Python distributions that support a hashlib.md5 with the usedforsecurity
-    # keyword can just use that md5 definition as-is
-    # See https://bugs.python.org/issue9216
-    #
-    # TODO(alee) Remove this wrapper when the minimum python version is bumped
-    # to 3.9 (which is the first upstream version to support this keyword)
-    # See https://docs.python.org/3.9/library/hashlib.html
-    md5 = hashlib.md5
-except TypeError:
-
-    def md5(string=b'', usedforsecurity=True):
-        """Return an md5 hashlib object without usedforsecurity parameter
-        For python distributions that do not yet support this keyword
-        parameter, we drop the parameter
-        """
-        return hashlib.md5(string)  # nosec
-
-
 def _calculate_data_hashes(data):
-    _md5 = md5(usedforsecurity=False)
+    _md5 = hashlib.md5(usedforsecurity=False)
     _sha256 = hashlib.sha256()
 
     if hasattr(data, 'read'):
@@ -417,7 +397,7 @@ class TinyDAG:
     def _start_traverse(self):
         """Initialize graph traversing"""
         self._run_in_degree = self._get_in_degree()
-        self._queue = queue.Queue()
+        self._queue: queue.Queue[str] = queue.Queue()
         self._done = set()
         self._it_cnt = len(self._graph)
 
@@ -427,8 +407,7 @@ class TinyDAG:
 
     def _get_in_degree(self):
         """Calculate the in_degree (count incoming) for nodes"""
-        _in_degree = dict()
-        _in_degree = {u: 0 for u in self._graph.keys()}
+        _in_degree: ty.Dict[str, int] = {u: 0 for u in self._graph.keys()}
         for u in self._graph:
             for v in self._graph[u]:
                 _in_degree[v] += 1
@@ -568,7 +547,7 @@ class Munch(dict):
 def munchify(x, factory=Munch):
     """Recursively transforms a dictionary into a Munch via copy."""
     # Munchify x, using `seen` to track object cycles
-    seen = dict()
+    seen: ty.Dict[int, ty.Any] = dict()
 
     def munchify_cycles(obj):
         try:
@@ -608,7 +587,7 @@ def unmunchify(x):
     """Recursively converts a Munch into a dictionary."""
 
     # Munchify x, using `seen` to track object cycles
-    seen = dict()
+    seen: ty.Dict[int, ty.Any] = dict()
 
     def unmunchify_cycles(obj):
         try:
